#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "splash_2bpp.h"

// control lines
#define	LCD_RESET	PC2
#define	LCD_RD    PC5
#define	LCD_WR    PC6
#define	LCD_RS	  PC7   
#define	LCD_CS	  PC4
#define LED_1     PD1
#define LED_2     PD2
#define BL_PWM		PB5

// macros for control lines
#define CLR_RS PORTC &= ~(1<<LCD_RS);
#define SET_RS PORTC |=  (1<<LCD_RS);

#define CLR_CS PORTC &= ~(1<<LCD_CS);
#define SET_CS PORTC |=  (1<<LCD_CS);

#define CLR_RESET PORTC &= ~(1<<LCD_RESET);
#define SET_RESET PORTC |=  (1<<LCD_RESET);

#define CLR_RW PORTC &= ~(1 << LCD_WR);
#define SET_RW PORTC |=  (1 << LCD_WR);
#define CLR_E PORTC &= ~(1 << LCD_RD);
#define SET_E PORTC |=  (1 << LCD_RD);

#define LED1_OFF PORTD &= ~(1 << LED_1);
#define LED1_ON  PORTD |=  (1 << LED_1);
#define LED2_OFF PORTD &= ~(1 << LED_2);
#define LED2_ON  PORTD |=  (1 << LED_2);

#define uint8 unsigned char
#define uint16 unsigned int
#define uint32 unsigned long

extern uint16 SectorsPerClust;
extern uint16 FirstDataSector; 
extern uint8 FAT32_Enable;

#define BACKLIGHT_ON	PORTB |=  (1 << BL_PWM);
#define BACKLIGHT_OFF	PORTB &= ~(1 << BL_PWM);

volatile unsigned char display[6400];  //12288 bytes

const unsigned char SplashText_Screen[30][41]=
  {"                                        ",  // 0
   "                                        ",  // 1
   "                                        ",  // 2
   "                                        ",  // 3
   "                                        ",  // 4
   "                                        ",  // 5
   "                                        ",  // 6
   "                                        ",  // 7
   "                                        ",  // 8
   "                                        ",  // 9
   "                                        ",  // 10
   "                                        ",  // 11
   "                                        ",  // 12
   "                                        ",  // 13
   "                                        ",  // 14
   "                                        ",  // 15
   "                                        ",  // 16
   "                                        ",  // 17
   "                                        ",  // 18
   "                                        ",  // 19
   "                               Built-in ",  // 20
   "                               5x7 font ",  // 21
   "                               on a     ",  // 22
   "                               separate ",  // 23
   "                               text     ",  // 24
   "                               layer.   ",  // 25
   "                               Can be   ",  // 26
   "                               combined ",  // 27
   "                               using AND",  // 28
   "                               OR or XOR"}; // 29



// ************************************************************************* //
void write_command(unsigned char command)
{
  PORTA=command;// write the command to the port

  CLR_CS;       // chip selected
  SET_RS;       // set to write to the control registers
  CLR_RW;       // writing 

  SET_E;       // clock in the command
	_delay_us(1);
 	CLR_E;       // bring write line low

  SET_CS;       // unselect chip
  SET_RS;       // set for data
}
// ************************************************************************* //
void write_data(unsigned char data)
{
  PORTA=data;// write the command to the port

  CLR_CS;       // chip selected
  CLR_RS;       // set to write to the control registers
  CLR_RW;       // writing

  SET_E;       // clock in the command
	_delay_us(1);
  CLR_E;       // bring write line low

  SET_CS;       // unselect chip
  SET_RS;       // set for data
}
// ************************************************************************* //
void clear_text_layer()
{
	unsigned short address=0;
  for(int row=0;row<30;row++)
	{
	  //CALL    POS1
	  write_command(0x46);
	  write_data(address&0x00FF);
	  write_data((address&0xFF00)>>8);
	  //Clear loop
	  write_command(0x42);
	  CLR_RS;
	  CLR_CS;
    for(int col=0;col<40;col++)
    {
			write_data(' ');
    }
		address+=80;
	}
	SET_CS;
}
// ************************************************************************* //
void initialize_display()
{
  CLR_E;
  CLR_RS;
  SET_CS;
  PORTA=0x00;

   SET_RESET;
   _delay_ms(10);
   CLR_RESET;
   _delay_ms(100);
   SET_RESET;
   _delay_ms(500);

  //Idle the control lines & reset the display



  //SYSTEM_SET:
  write_command(0x40);

  write_data(0x30); //     REG[00h] Memory Configuration Register
  // 0011 0000  
  // |||| ||||-- bit 0: Character Generator Select (M0)
  // |||| |||      0=the internal CGROM is selected.
  // |||| |||      1=the internal CGRAM is selected.
  // |||| |||--- bit 1 Reserved = 0
  // |||| ||---- bit 2 Character Height (M2)
  // |||| |        When this bit = 0, the character height is 8 pixels.
  // |||| |        When this bit = 1, the character height is 16 pixels.
  // |||| |----- bit 3 Panel Drive Select (W/S)
  // ||||          When this bit = 0, a single panel drive is selected.
  // ||||          When this bit = 1, a dual panel drive is selected.
  // ||||
  // ||||------- bit 4 Reserved = 1
  // |||-------- bit 5 Screen Origin Compensation (IV)
  // ||            When this bit = 0, screen origin compensation is done.
  // ||            When this bit = 1, screen origin compensation is not done.
  // ||--------- bit 6 unused = 0
  // |---------- bit 7 unused = 0
  
  write_data(0x87); //FX=  REG[01h] Horizontal Character Size Register
  // 1000 0111  
  // |||| ||||-- Horizontal Character Size (FX) bits 0-3
  // ||||          These bits define the horizontal size, or width, of each character, in pixels.
  // |||| 	   REG[01h] bits 3-0 = Horizontal Character Size in pixels - 1
  // ||||------- bit 4,5,6 unused = 0
  // |---------- bit 7 MOD
  //		   When this bit = 0, 16-line AC drive is selected.
  //		   When this bit = 1, two-frame AC drive is selected.

  write_data(0x07); //FY=  REG[02h] Vertical Character Size Register
  // 0000 0111  
  // |||| ||||-- Vertical Character Size (FY) bits 0-3
  // ||||          These bits define the vertical size, or height, of each character, in pixels.
  // |||| 	   REG[02h] bits 3-0 = Vertical Character Size in pixels - 1
  // ||||------- bits 4-7 unused = 0

  write_data(79); //C/R= REG[03h] Character Bytes Per Row Register
  // These bits determine the size of each character row (or display line), in bytes,
  // to a maximum of 239. The value of these bits is defined in terms of C/R
  // REG[03h] bits 7-0 = ([C/R] x bpp) - 1  (bpp == bits per pixel)
  //
  //   C/R =(320 dots / 8 dots/character) = 40
  //   2 bpp = 80
  //   REG[03h] = [C/R] x bpp) - 1 = 79

  write_data(81); //TC/R=REG[04h] Total Character Bytes Per Row Register
  // These bits set the length of one line, including horizontal blanking, in bytes, 
  // to a maximum of 255. The value of these bits is defined in terms of TC/R TC/R 
  // can be adjusted to hold the frame period constant and minimize jitter for any 
  // given main oscillator frequency, fosc.
  // REG[04h] bits 7-0 = [TC/R] + 1
  // TC/R must be programmed such that the following formulas are valid.
  // [C/R] + 2 <= [TC/R]
  // 0 <= [TC/R] <= 255

  write_data(239); //L/F= REG[05h] Frame Height Register
  // These bits determine the frame height, in lines. The maximum frame height is 
  // 256 lines.
  // REG[05h] bits 7-0 = frame height in lines - 1.
  // 240 lines in this module, so REG[05h] = 240-1 = 239

  write_data(0x50); //APL= REG[06h] Horizontal Address Range Register 0
  write_data(0x00); //APH= REG[07h] Horizontal Address Range Register 1
  // These bits define the horizontal address range of the virtual screen. 
  // The maximum value for this register is 7FFFh.
  // REG[07h] bits 7-0, REG[06h] bits 7-0 = Addresses per line
  // Number of bytes used for one line = 80 = 0x0050
 		   
  //SCROLL:
  write_command(0x44);
  write_data(0x00); // REG[0Bh] Screen Block 1 Start Address Register 0
  write_data(0x00); // REG[0Ch] Screen Block 1 Start Address Register 1
  // Screen Block 1 Start Address (SAD1) bits [15:0]
  // These bits determine the memory start address of screen block 1.

  write_data(239); // REG[0Dh] Screen Block 1 Size Register
  // These bits determine the size of screen block 1, in lines.
  // REG[0Dh] bits 7-0 = screen block 1 size in number of lines - 1
  // 240 lines - 1 = 239

  write_data(0x60); // REG[0Eh] Screen Block 2 Start Address Register 0
  write_data(0x09); // REG[0Fh] Screen Block 2 Start Address Register 1
  // These bits determine the memory start address of screen block 2.
  // 0x0960

  write_data(239); // REG[10h] Screen Block 2 Size Register
  // These bits determine the size of screen block 2, in lines.
  // REG[10h] bits 7-0 = screen block 2 size in number of lines - 1
  // 240 lines - 1 = 239

  write_data(0x00); // REG[11h] Screen Block 3 Start Address Register 0
  write_data(0x00); // REG[12h] Screen Block 3 Start Address Register 1
  // These bits determine the memory start address of screen block 3.
  // 0x0000

  write_data(0x00); // REG[13h] Screen Block 4 Start Address Register 0
  write_data(0x00); // REG[14h] Screen Block 4 Start Address Register 1
  // These bits determine the memory start address of screen block 4.
  // 0x0000

  //HDOT_SCR:
  write_command(0x5A); // REG[1Bh] Horizontal Pixel Scroll Register
  write_data(0x00);  //NO SCROLL
  // 0000 0000  
  // |||| ||||-- Horizontal Pixel Scroll bit 1 - 3
  // |||| |----- bit 3-7 unused = 0
  // These bits specify the number of horizontal pixels to scroll the display. 
  // The character bytes per row (C/R), REG[03h] bits 7-0, must be set to one 
  // more than the actual number of horizontal characters before using horizontal 
  // pixel scroll. Smooth scrolling can be simulated by repeatedly changing the 
  // value of REG[1Bh] bits 2-0.

  //OVERLAY:  (Note: Screen blocks 2 and 4 can display graphics only.)
  write_command(0x5B); // REG[18h] Overlay Register
  
  write_data(0x01);   //GRAPHIC & XOR
  // 0000 0001  
  // |||| ||||-- bit 0 Layer Composition Method (MX) bit 1
  // |||| |||--- bit 1 Layer Composition Method (MX) bit 2
  // |||| ||       These bits select the layered screen composition method, 
  // |||| ||       which can be OR, AND, or Exclusive-OR.
  // |||| ||       
  // |||| ||---- bit 2 Screen Block 1 Display Mode (DM0)
  // |||| |        This bit determines the display mode for screen block 1.
  // |||| |	   When this bit = 0, screen block 1 is configured for text mode.
  // |||| |        When this bit = 1, screen block 1 is configured for graphics mode.
  // |||| |
  // |||| |----- bit 3 Screen Block 3 Display Mode (DM1)
  // ||||          This bit determines the display mode for screen block 3.
  // ||||          When this bit = 0, screen block 3 is configured for text mode.
  // ||||          When this bit = 1, screen block 3 is configured for graphics mode.
  // ||||
  // ||||------- bit 4 3 Layer Overlay Select (OV)
  // |||           This bit determines how many layers are used when graphics mode is enabled. 
  // |||           - For mixed text and graphics, this bit must be set to 0.
  // |||           When this bit = 0, two layers are used.
  // |||           When this bit = 1, three layers are used.
  // |||
  // |||-------- bit 5-7 unused = 0

  //CSR_FORM:
  write_command(0x5D); 
  write_data(0x06); // REG[15h] Cursor Width Register
  // 0000 0110  
  // |||| ||||-- bit 0 Cursor Width (CRX) bit 0-3
  // ||||          These bits specify the width (or horizontal size) of the cursor, in pixels from 
  // |||| 	       the character origin 
  // ||||          REG[15h] bits 3-0 = cursor width in pixels - 1
  // ||||
  // ||||------- bit 4-7 unused = 0

  write_data(0x86); // REG[16h] Cursor Height Register
  // 1000 0110  
  // |||| ||||-- bit 0 Cursor Height (CRY) bit 0
  // |||| |||--- bit 1 Cursor Height (CRY) bit 1
  // |||| ||---- bit 2 Cursor Height (CRY) bit 2
  // |||| |----- bit 3 Cursor Height (CRY) bit 3
  // ||||          For an underscore cursor (REG[16h] bit 7 = 0), these bits set the location 
  // ||||          of the cursor, in lines from the character origin.
  // ||||          For a block cursor (REG[16h] bit 7 = 1), these bits set the height 
  // ||||          (or vertical size) of the cursor, in lines from the character origin.
  // ||||	   REG[16h] bits 3-0 = cursor height in lines - 1
  // ||||
  // ||||------- bit 4-7 unused = 0
  // This bit determines the cursor mode. When graphics mode is selected, 
  // this bit must be set to 1.
  // When this bit = 0, an underscore cursor ( _ ) is selected.
  // When this bit = 1, a block cursor (  ) is selected.
  
  //CSR_DIR:
  write_command(0x4C); // REG[17h] Cursor Shift Direction Register  -- Default = 00h
  // 0100 11dd  
  // |||| ||||-- bit 1:0
  // |||| ||         0 0 Right (command 0x4C)
  // |||| ||         0 1 Left  (command 0x4D)
  // |||| ||         1 0 Up    (command 0x4E)
  // |||| ||         1 1 Down  (command 0x4F)
  // ||||-||---- bit 2-7 (command 0x4C base)

  //DISP_ON:
  write_command(0x59);
  write_data(0x14); // REG[0Ah] Display Attribute Register values
  // 0001 0000  
  // |||| ||||-- bit 0 Cursor Attribute (FC) bit 0
  // |||| |||--- bit 1 Cursor Attribute (FC) bit 1
  // |||| ||       These bits control the cursor and set the flash rate -- 70% duty cycle (70% on, 30% off):
  // |||| ||       0 0 OFF (Blank)
  // |||| ||       0 1 ON  No Flashing
  // |||| ||       1 0 ON  Flash at fFR/32 Hz (approx. 2 Hz)
  // |||| ||       1 1 ON  Flash at fFR/64 Hz (approx. 1 Hz)
  // |||| ||       NOTE:  When the cursor is disabled, a write to memory automatically enables the cursor and
  // |||| ||              places the cursor at the next memory location. A read from memory does not enable the
  // |||| ||              cursor, however, it still places the cursor at the next memory location.
  // |||| ||
  // |||| ||---- bit 2 SAD1 Attribute (FP 1-0) bit 0
  // |||| |----- bit 3 SAD1 Attribute (FP 1-0) bit 1
  // ||||          These bits control the attributes of the third screen block (SAD1):
  // ||||          0 0 OFF (Blank)
  // ||||          0 1 ON  No Flashing
  // ||||          1 0 ON  Flash at fFR/32 Hz (approx. 2 Hz)
  // ||||          1 1 ON  Flash at fFR/4 Hz (approx. 16 Hz)
  // ||||
  // ||||------- bit 4 SAD2 Attribute (FP 3-2) bit 0
  // |||-------- bit 5 SAD2 Attribute (FP 3-2) bit 1
  // ||            These bits control the attributes of the third screen block (SAD2):
  // ||            0 0 OFF (Blank)
  // ||            0 1 ON  No Flashing
  // ||            1 0 ON  Flash at fFR/32 Hz (approx. 2 Hz)
  // ||            1 1 ON  Flash at fFR/4 Hz (approx. 16 Hz)
  // ||            
  // ||--------- bit 6 SAD3 Attribute (FP 5-4) bit 0
  // |---------- bit 7 SAD3 Attribute (FP 5-4) bit 1
  //               These bits control the attributes of the third screen block (SAD3):
  //               0 0 OFF (Blank)
  //               0 1 ON  No Flashing
  //               1 0 ON  Flash at fFR/32 Hz (approx. 2 Hz)
  //               1 1 ON  Flash at fFR/4 Hz (approx. 16 Hz)

  //Grayscale
  write_command(0x60);
  write_data(0x01); //REG[20h] Bit-Per-Pixel Select Register
  // 0000 11dd  
  // |||| ||||-- bit 1:0
  // |||| ||         0 0 Bits per pixel = 1
  // |||| ||         0 1 Bits per pixel = 2
  // |||| ||         1 0 Bits per pixel = 4
  // |||| ||         1 1 reserved
  // ||||-||---- bit 2-7 unused = 0

clear_text_layer();
}
// ************************************************************************* //
void display_splash()
{
	for(int i=0;i<6400;i++)
		display[i]=0x00;

  //CALL    POS1
  write_command(0x46);
  write_data(0x60);
  write_data(0x09);

  write_command(0x42);
  CLR_RS;
  CLR_CS;

	memcpy_P(&display, &gImage_splash_2bpp[0], sizeof(display));

  int row=0;
	int col=0;

  for(row=0;row<6400;row++)
	{
			write_data(display[row]);//((display[6399-row]<<6)|((display[6399-row]<<4)|((display[6399-row]<<2)|((display[6399-row]<<0));
	}

	memcpy_P(&display, &gImage_splash_2bpp[6400], sizeof(display));

	row=0;
	col=0;
  for(row=0;row<6400;row++)
	{
			write_data(display[row]);//((display[6399-row]<<6)|((display[6399-row]<<4)|((display[6399-row]<<2)|((display[6399-row]<<0));
	}

 	memcpy_P(&display, &gImage_splash_2bpp[12800], sizeof(display));

	row=0;
	col=0;
  for(row=0;row<6400;row++)
	{
			write_data(display[row]);//((display[6399-row]<<6)|((display[6399-row]<<4)|((display[6399-row]<<2)|((display[6399-row]<<0));
	}

	unsigned short address=0;
  for(row=0;row<30;row++)
	{
	  //CALL    POS1
	  write_command(0x46);
	  write_data(address&0x00FF);
	  write_data((address&0xFF00)>>8);
	  //Clear loop
	  write_command(0x42);
	  CLR_RS;
	  CLR_CS;
    for(col=0;col<40;col++)
    {
			write_data(SplashText_Screen[row][col]);
    }
		address+=80;
	}
	SET_CS;
}
// ************************************************************************* //
int main()
{
  DDRC=0xFF;    // all pins as outputs
  DDRA=0xFF;    // all pins as outputs
	DDRD |= 0x06;	// Switches as inputs, LEDs as outputs
	DDRB=0xFF;
  PORTA=0xFF;   // data lines low
  PORTC=0xFF;   // control and other lines high
  PORTD=0xF8;   // pull ups on the keys, remaining pins low

	BACKLIGHT_ON;

	_delay_ms(100);

	// Initialize the display
	initialize_display();

		display_splash();

	while (1)
	{
			// uncomment for fun
//		display_splash();
//		_delay_ms(10000);
//		clear_text_layer();
//		display_orion();
//		_delay_ms(10000);
//		display_rocket();
//		_delay_ms(10000);
		LED2_OFF;
	   _delay_ms(100);
		LED2_ON;
	   _delay_ms(100);
		LED2_OFF;
	}
	return 0;
}

